{$A+,B-,D+,E+,F-,G+,I+,L+,N+,O-,P-,Q+,R+,S+,T-,V+,X+}
unit EgerKez;

interface
uses  Dos;

const
  egBalGomb     = 1;
  egJobbGomb    = 2;
  egKozepsoGomb = 4;

{ Kperny zemmd lekrdezse }
function SzovegesMod:Boolean;          { Szveges zemmd esetn True }

{ Egr alaphelyzetbe lltsa }
function EgerReset:Boolean;            { True, ha az egr hasznlhat }

{ Gombok szmnak lekrdezse }
function EgerGombSzam:Byte;            { Egrgombok szma }

{ Lthat-e az egr }
function EgerLathato:Boolean;          { True, ha az egr lthat }

{ Hasznlhat-e az egr }
function EgerMukodik:Boolean;          { True, ha az egr hasznlht }

{ Visszajelzs a gombok llapotrl }
function EgerGombLenyomva:Byte;        { A gombok llapota                   }
                                       { Pl.: a bal gomb le van nyomva, ha   }
                                       { (EgerGombLenyomva and egBalGomb)<>0 }

{ Bal gomb le van-e nyomva a hvs pillanatban}
function EgerBalGomb:Boolean;          { True, ha a bal gomb le van nyomva }

{ Kzps gomb le van-e nyomva a hvs pillanatban}
function EgerKozepsoGomb:Boolean;      { True, ha a kzps gomb le van nyomva }

{ Jobb gomb le van-e nyomva a hvs pillanatban}
function EgerJobbGomb:Boolean;         { True, ha a jobb gomb le van nyomva }

function EgerKattintas:Byte;           { Ha a a hvs pillanatban volt    }
                                       { lenyomott gomb, akkor az utoljra }
                                       { felengedett gomb, egybknt 0     }

{ Egr pozci lekrdezse }
procedure EgerPozicio(var x,           { o   az egr oszlop pozcija }
                          y:Word);     { o   az egr sor pozcija }

{ Egr lthatv ttele }
procedure EgerBe;

{ Egr lthatatlann ttele }
procedure EgerKi;

{ Egr mozgatsa adott pozcira }
procedure EgerXY(x,                    { i   oszlop koordinta }
                 y:Word);              { i   sor koordinta }

{ Egr mozgsterletnek meghatrozsa }
procedure EgerAblak(x1,y1,             { i   bal fels sarok koordinti }
                    x2,y2:Word);       { i   jobb als sarok koordinti }

{ Megjegyzs: a bal fels sarok koordintja szveges zemmdban (1,1),  }
{             grafikus zemmdban (0,0).                                 }

implementation

const
  gombszam :Byte = 0;
  lathato :Integer = 0;
  mukodik :Boolean = False;

{----[ SzovegesMod ]---------------------------------------------------------}

function SzovegesMod:Boolean;
begin
  SzovegesMod:=Mem[$0040:$0049] in [0..3,7];
end;

{----[ EgerReset ]-----------------------------------------------------------}

{ Inicializlja vagy alaphelyzetbe lltja az egeret, meghatrozza az egr-
  gombok szmt.
  Visszaadott rtk: True, ha hasznlhat az egrmegszakts }
function EgerReset:Boolean;
var
  r :Registers;
begin
  r.ax:=0;
  Intr($33,r);
  gombszam:=r.bl;
  lathato:=0;
  mukodik:=r.al<>0;
  if mukodik then begin
    r.cx:=0;
    r.dx:=0;
    r.ax:=4;
    Intr($33,r);
  end;
  EgerReset:=mukodik;
end;

{----[ EgerGombSzam ]--------------------------------------------------------}

{ Visszaadott rtk: az egrgombok szma }
function EgerGombSzam:Byte;
begin
  EgerGombSzam:=gombszam;
end;

{----[ EgerLathato ]---------------------------------------------------------}

{ Visszaadott rtk: True, ha az egr lthat }
function EgerLathato:Boolean;
begin
  EgerLathato:=lathato>0;
end;

{----[ EgerMukodik ]---------------------------------------------------------}

{ Visszaadott rtk: True, ha az egr hasznalhato }
function EgerMukodik:Boolean;
begin
  EgerMukodik:=mukodik;
end;

{----[ EgerGombLenyomva ]----------------------------------------------------}

{ Visszaadott rtk: az egrgombok llapota
  Bit0 - bal gomb
  Bit1 - jobb gomb
  Bit2 - kzps gomb
  A bitek rtke 0, ha ilyen gomb nem ltezik, vagy nincs lenyomva. }
function EgerGombLenyomva:Byte;
var
  r :Registers;
begin
  r.ax:=5;
  Intr($33,r);
  EgerGombLenyomva:=r.al;
end;

{----[ EgerBalGomb ]---------------------------------------------------------}

{ Bal gomb le van-e nyomva }
function EgerBalGomb:Boolean;
begin
  EgerBalGomb:=(EgerGombLenyomva and egBalGomb)<>0;
end;

{----[ EgerKozepsoGomb ]-----------------------------------------------------}

{ Kzps gomb le van-e nyomva }
function EgerKozepsoGomb:Boolean;
begin
  EgerKozepsoGomb:=(EgerGombLenyomva and egKozepsoGomb)<>0;
end;

{----[ EgerJobbGomb ]--------------------------------------------------------}

{ Jobb gomb le van-e nyomva }
function EgerJobbGomb:Boolean;
begin
  EgerJobbGomb:=(EgerGombLenyomva and egJobbGomb)<>0;
end;

{----[ EgerKattintas ]-------------------------------------------------------}

{ Visszaadott rtk: ha trtnt kattints, a megfelel egrgombkonstans,
  egybknt 0 }
function EgerKattintas:Byte;
var
  lenyomva,utolso :Byte;
begin
  lenyomva:=0;
  repeat
    utolso:=lenyomva;
    lenyomva:=EgerGombLenyomva;
  until lenyomva=0;
  EgerKattintas:=utolso;
end;

{----[ EgerPozicio ]---------------------------------------------------------}

{ Visszaadja az egr sor s oszlop pozcijt }
procedure EgerPozicio(var x,y:Word);
var
  r :Registers;
begin
  r.ax:=3;
  Intr($33,r);
  if SzovegesMod then begin
    r.cx:=(r.cx shr 3)+1;
    r.dx:=(r.dx shr 3)+1;
  end;
  x:=r.cx;
  y:=r.dx;
end;

{----[ EgerBe ]--------------------------------------------------------------}

{ Lthatv teszi az egeret }
procedure EgerBe;
var
  r :Registers;
begin
  r.ax:=1;
  Intr($33,r);
  if lathato<1 then Inc(lathato);
end;

{----[ EgerKi ]--------------------------------------------------------------}

{ Lthatatlann teszi az egeret }
procedure EgerKi;
var
  r :Registers;
begin
  r.ax:=2;
  Intr($33,r);
  Dec(lathato);
end;

{----[ EgerXY ]--------------------------------------------------------------}

{ Az x oszlop y sorba mozgatja az egeret }
procedure EgerXY(x,y:Word);
var
  r :Registers;
begin
  if SzovegesMod then begin
    Dec(x);
    x:=x shl 3;
    Dec(y);
    y:=y shl 3;
  end;
  r.cx:=x;
  r.dx:=y;
  r.ax:=4;
  Intr($33,r);
end;

{----[ EgerAblak ]-----------------------------------------------------------}

{ Egr mozgsterletnek belltsa }
procedure EgerAblak(x1,y1,x2,y2:Word);
var
  r :Registers;
begin
  if SzovegesMod then begin
    Dec(x1);
    x1:=x1 shl 3;
    Dec(y1);
    y1:=y1 shl 3;
    Dec(x2);
    x2:=x2 shl 3;
    Dec(y2);
    y2:=y2 shl 3;
  end;
  r.cx:=x1;
  r.dx:=x2;
  r.ax:=7;
  Intr($33,r);
  r.cx:=y1;
  r.dx:=y2;
  r.ax:=8;
  Intr($33,r);
end;

{----[ EgerKurzor ]----------------------------------------------------------}

type
  TEgerKurzor = record
    csucsx,csucsy :Word;
    kepernyomaszk,kurzormaszk :array[0..15] of Word;
  end;

{ Egrkurzor belltsa }
procedure EgerKurzor(const kurzor:TEgerKurzor);
var
  r :Registers;
begin
  with kurzor do begin
    r.bx:=csucsx;
    r.cx:=csucsy;
    r.es:=Seg(kepernyomaszk);
    r.dx:=Ofs(kepernyomaszk);
  end;
  r.ax:=9;
  Intr($33,r);
end;

{----[ EgerKurzor ]----------------------------------------------------------}

type
 TEgerEljaras = procedure;

{ Esemnykezel eljrs belltsa. Ha az egrrel valamilyen mveletet vg-
  znk (mozgats, gomb lenyomsa vagy felengedse), az egrmeghajt meghv-
  ja ezt a rutint. }
procedure EgerEljaras(const eljaras:TEgerEljaras);
var
  r :Registers;
begin
  r.cx:=$ffff;
  r.es:=Seg(eljaras);
  r.dx:=Ofs(eljaras);
  r.ax:=12;
  Intr($33,r);
end;

end.